/**
  *  \file server/serverconsole.cc
  *  \brief Server: Console
  *
  *  This implements a PCC2 console module for use in the server. The
  *  normal console module (mod_base_console) cannot be used here to
  *  not interfere with regular console operation; our stdin/stdout
  *  are used to communicate with c2router.
  */

#include "server/serverconsole.h"
#include "util/console.h"
#include "server/serverio.h"

static bool console_enabled = false;
static int console_missed_lines = 0;

static void
serverConsoleHandler(const Console& con, int n)
{
    console_missed_lines += n;
    if (console_enabled) {
        // Make sure we don't want to write more than there is stored
        // in case the console was disabled for a long time.
        if (con.getScrollbackFill() < console_missed_lines) {
            console_missed_lines = con.getScrollbackFill();
        }

        // Dump all lines on stdout, prefixed by "//" so they don't
        // disturb JSON output if it gets mixed in accidentally.
        while (console_missed_lines > 0) {
            --console_missed_lines;
            writeClient("// " + con.getLine(console_missed_lines).text);
        }
    }
}

/** Initialize server console. */
void
initServerConsole()
{
    console.console_changed.add(serverConsoleHandler);
}

/** Set server console state. The console can be disabled so it only
    collects data in its internal buffer. When enabled, it writes data
    directly to stdout. */
void
setServerConsoleState(bool enable)
{
    console_enabled = enable;
    if (enable) {
        serverConsoleHandler(console, 0);
    }
}
